@@ -20,13 +20,16 @@ import threading
 import ipykernel
 from ipykernel.ipkernel import IPythonKernel
 from ipykernel.zmqshell import ZMQInteractiveShell
+from traitlets.config.loader import LazyConfigValue
 
 # Local imports
 from spyder_kernels.py3compat import TEXT_TYPES, to_text_string
 from spyder_kernels.comms.frontendcomm import FrontendComm
 from spyder_kernels.py3compat import PY3, input
-from spyder_kernels.utils.misc import (
+from spyder_kernels.utils.iofuncs import iofunctions
+from spyder_kernels.utils.mpl import (
     MPL_BACKENDS_FROM_SPYDER, MPL_BACKENDS_TO_SPYDER, INLINE_FIGURE_FORMATS)
+from spyder_kernels.utils.nsview import get_remote_data, make_remote_view
 
 
 # Excluded variables from the Variable Explorer (i.e. they are not
@@ -63,6 +66,7 @@ class SpyderKernel(IPythonKernel):
     def __init__(self, *args, **kwargs):
         super(SpyderKernel, self).__init__(*args, **kwargs)
 
+        self.comm_manager.get_comm = self._get_comm
         self.frontend_comm = FrontendComm(self)
 
         # All functions that can be called through the comm
@@ -103,7 +107,6 @@ class SpyderKernel(IPythonKernel):
         self.namespace_view_settings = {}
         self._pdb_obj = None
         self._pdb_step = None
-        self._do_publish_pdb_state = True
         self._mpl_backend_error = None
         self._running_namespace = None
         self._pdb_input_line = None
@@ -155,7 +158,6 @@ class SpyderKernel(IPythonKernel):
         * 'numpy_type' is its Numpy type (if any) computed with
           `get_numpy_type_string`.
         """
-        from spyder_kernels.utils.nsview import make_remote_view
 
         settings = self.namespace_view_settings
         if settings:
@@ -170,8 +172,6 @@ class SpyderKernel(IPythonKernel):
         Get some properties of the variables in the current
         namespace
         """
-        from spyder_kernels.utils.nsview import get_remote_data
-
         settings = self.namespace_view_settings
         if settings:
             ns = self._get_current_namespace()
@@ -200,7 +200,6 @@ class SpyderKernel(IPythonKernel):
     def get_value(self, name):
         """Get the value of a variable"""
         ns = self._get_current_namespace()
-        self._do_publish_pdb_state = False
         return ns[name]
 
     def set_value(self, name, value):
@@ -232,11 +231,9 @@ class SpyderKernel(IPythonKernel):
         In the other hand, with 'overwrite=False', a new variable will be
         created with a sufix starting with 000 i.e 'var000' (default behavior).
         """
-        from spyder_kernels.utils.iofuncs import iofunctions
         from spyder_kernels.utils.misc import fix_reference_name
 
         glbs = self._mglobals()
-
         load_func = iofunctions.load_funcs[ext]
         data, error_message = load_func(filename)
 
@@ -259,9 +256,6 @@ class SpyderKernel(IPythonKernel):
 
     def save_namespace(self, filename):
         """Save namespace into filename"""
-        from spyder_kernels.utils.nsview import get_remote_data
-        from spyder_kernels.utils.iofuncs import iofunctions
-
         ns = self._get_current_namespace()
         settings = self.namespace_view_settings
         data = get_remote_data(ns, settings, mode='picklable',
@@ -290,16 +284,12 @@ class SpyderKernel(IPythonKernel):
         return self._do_complete(code, cursor_pos)
 
     def publish_pdb_state(self):
-        """
-        Publish Variable Explorer state and Pdb step through
-        send_spyder_msg.
-        """
-        if self._pdb_obj and self._do_publish_pdb_state:
+        """Publish Pdb state."""
+        if self._pdb_obj:
             state = dict(namespace_view = self.get_namespace_view(),
                          var_properties = self.get_var_properties(),
                          step = self._pdb_step)
             self.frontend_call(blocking=False).pdb_state(state)
-        self._do_publish_pdb_state = True
 
     def set_spyder_breakpoints(self, breakpoints):
         """
@@ -456,7 +446,7 @@ class SpyderKernel(IPythonKernel):
         """
         Set inline print figure bbox inches.
 
-        The change is done by updating the ´rint_figure_kwargs' config dict.
+        The change is done by updating the 'print_figure_kwargs' config dict.
         """
         from IPython.core.getipython import get_ipython
         config = get_ipython().kernel.config
@@ -468,6 +458,14 @@ class SpyderKernel(IPythonKernel):
         bbox_inches_dict = {
             'bbox_inches': 'tight' if bbox_inches else None}
         print_figure_kwargs.update(bbox_inches_dict)
+
+        # This seems to be necessary for newer versions of Traitlets because
+        # print_figure_kwargs doesn't return a dict.
+        if isinstance(print_figure_kwargs, LazyConfigValue):
+            figure_kwargs_dict = print_figure_kwargs.to_dict().get('update')
+            if figure_kwargs_dict:
+                print_figure_kwargs = figure_kwargs_dict
+
         self._set_config_option(
             'InlineBackend.print_figure_kwargs', print_figure_kwargs)
 
@@ -720,6 +718,7 @@ class SpyderKernel(IPythonKernel):
         """
         import traceback
         from IPython.core.getipython import get_ipython
+        import matplotlib
 
         generic_error = (
             "\n" + "="*73 + "\n"
@@ -732,12 +731,16 @@ class SpyderKernel(IPythonKernel):
 
         error = None
         try:
+            # This prevents Matplotlib to automatically set the backend, which
+            # overrides our own mechanism.
+            matplotlib.rcParams['backend'] = 'Agg'
+
+            # Set the backend
             get_ipython().run_line_magic(magic, backend)
         except RuntimeError as err:
             # This catches errors generated by ipykernel when
             # trying to set a backend. See issue 5541
             if "GUI eventloops" in str(err):
-                import matplotlib
                 previous_backend = matplotlib.get_backend()
                 if not backend in previous_backend.lower():
                     # Only inform about an error if the user selected backend
@@ -750,7 +753,6 @@ class SpyderKernel(IPythonKernel):
                         "in use.\n\n"
                         "Your backend will be {0}".format(previous_backend)
                     )
-                del matplotlib
             # This covers other RuntimeError's
             else:
                 error = generic_error.format(traceback.format_exc())
@@ -830,3 +832,16 @@ class SpyderKernel(IPythonKernel):
                 get_ipython().run_line_magic('reload_ext', 'wurlitzer')
             except Exception:
                 pass
+
+    def _get_comm(self, comm_id):
+        """
+        We need to redefine this method from ipykernel.comm_manager to
+        avoid showing a warning when the comm corresponding to comm_id
+        is not present.
+
+        Fixes spyder-ide/spyder#15498
+        """
+        try:
+            return self.comm_manager.comms[comm_id]
+        except KeyError:
+            pass
